@@ -9,8 +9,12 @@
 #' @param alpha Significance level
 #' @param padj Adjustment to make to pvalues for significance. Will be able to
 #' use anything from \code{p.adjust} function, but for now, just "oleson"
+#' @param permutation Boolean indicating whether to use permutation testing rather
+#' thank adjusting alpha to control FWER. WARNING: This option is very much in beta testing
+#' and not recommended for general use. Also not available for paired tests or difference of difference
 #' @param cores Number of cores to use in parallel. Default is zero, which
 #' uses half of what is available.
+#' @param permAddVar Boolean. Add observed within-subject variability for permutation testing?
 #' @param ... not used
 #'
 #' @details The formula is the only tricky part of this. There will be a minor
@@ -34,7 +38,7 @@
 #'
 #' ### Bootstrapped difference of curves
 #'
-#' This illustrates the case in which we are taking a simple bootstraped difference
+#' This illustrates the case in which we are taking a simple bootstrapped difference
 #' between two curves within a single group
 #'
 #' If only one group was provided in \code{bdotsFit}, we can take the bootstrapped
@@ -91,73 +95,131 @@
 #'
 #' ## fit <- bdotsFit(cohort_unrelated, ...)
 #'
-#' boot1 <- bdotsBoot(formula = diffs(Fixations, LookType(Cohort, Unrelated_Cohort)) ~ Group(50, 65),
-#'                    bdObj = fit,
-#'                    N.iter = 1000,
-#'                    alpha = 0.05,
-#'                    p.adj = "oleson",
-#'                    cores = 4)
+#' boot1 <- bboot(formula = diffs(Fixations, LookType(Cohort, Unrelated_Cohort)) ~ Group(50, 65),
+#'                bdObj = fit,
+#'                N.iter = 1000,
+#'                alpha = 0.05,
+#'                p.adj = "oleson",
+#'                cores = 4)
 #'
-#' boot2 <- bdotsBoot(formula = Fixations ~ Group(50, 65) + LookType(Cohort),
-#'                    bdObj = fit,
-#'                    N.iter = 1000,
-#'                    alpha = 0.05,
-#'                    p.adj = "oleson",
-#'                    cores = 4)
+#' boot2 <- bboot(formula = Fixations ~ Group(50, 65) + LookType(Cohort),
+#'                bdObj = fit,
+#'                N.iter = 1000,
+#'                alpha = 0.05,
+#'                p.adj = "oleson",
+#'                cores = 4)
 #' }
 #'
 #' @import data.table
 #' @export
-bdotsBoot <- function(formula,
-                      bdObj,
-                      Niter = 1000,
-                      alpha = 0.05,
-                      padj = "oleson",
-                      cores = 0, ...) {
-  
+bboot <- function(formula,
+                  bdObj,
+                  Niter = 1000,
+                  alpha = 0.05,
+                  padj = "oleson",
+                  permutation = TRUE,
+                  permAddVar = TRUE,
+                  cores = 0, ...) {
+
+
+  ## These are secret shortcuts used for methods paper but not intended
+  ## for distribution or general use
+
+  ## singleMeans - 'original' bdots implementation, that which we are showing is faulty
+  ## skipDist - only useful for permutation method, skips construction of bootstrapped distribution
+
+  dd <- list(...) # Evaluate arguments in dots
+  singleMeans <- ifelse(is.null(dd[["singleMeans"]]), FALSE, dd[["singleMeans"]])
+  skipDist <- ifelse(!is.null(dd[["skipDist"]]) & permutation, dd[["skipDist"]], FALSE)
+
+
+  ## Rest of the function as normal
   if (cores < 1) cores <- detectCores()/2
-  
+
   if (any(bdObj[['fitCode']] == 6)) {
     warning("Some observations had NULL gnls fits. These and their pairs will be removed")
     bdObj <- bdRemove(bdObj, fitCode = 6, removePairs = TRUE)
   }
-  
+
   prs <- bootParser(formula, bdObj)
   bdObj <- bootGroupSubset(prs, bdObj)
   innerDiff <- prs[["innerDiff"]] # unnamed char vec
   outerDiff <- prs[["outerDiff"]] # unnamed char vec
-  
+
   curveGrps <- setNames(prs[['subargs']], prs[['subnames']])
   curveFun <- makeCurveFun(bdObj)
-  
+
   ## Next, we want to get a bootstrapped distribution for each of the groups
   splitGroups <- split(bdObj, by = c(innerDiff, outerDiff)) # ok even if null
-  
-  ## Make this parallel (maybe later)
-  if (Sys.info()['sysname'] == "Darwin") {
-    cl <- makePSOCKcluster(cores, setup_strategy = "sequential")
+
+  ## This cannot stay here forever, its only here until methods published
+  ## Basically only keep !singleMeans
+  #browser()
+
+  if (skipDist) {
+    # just skip doing this
+    curveList <- NULL
+    ip <- NULL
+  } else if (singleMeans) {
+    ## THIS IS RUNNING UNDER SINGLE MEANS ASSUMPTION
+    # pulled from old bdots in curveBooter_SINGLEMEAN.R
+    curveList <- curveBooter_sm(bdObj,
+                             outerDiff = outerDiff,
+                             innerDiff = innerDiff,
+                             N.iter = Niter,
+                             curveFun = curveFun)
+    ip <- curveList[['diff']][['paired']]
   } else {
-    cl <- makePSOCKcluster(cores)
+    ## This is the standard thing to happen, delete everything else in this if condition
+    ##  parallel moved inside of this function
+    groupDists <- createGroupDists(splitGroups, prs, b = Niter, cores)
+
+    ## This is where we construct inner/outer groups
+    curveList <- createCurveList(groupDists, prs, splitGroups) # this is whats creates diff
+    ip <- curveList[['diff']][['paired']]
   }
-  invisible(clusterEvalQ(cl, {library(bdots)}))
-  groupDists <- parLapply(cl, splitGroups, getBootDist, b = Niter)
-  
-  stopCluster(cl)
-  
-  ## Next, we need to make inner/outer diff list
-  # (ideally matching old bdots, at least for now)
-  curveList <- createCurveList(groupDists, prs, splitGroups)
-  ip <- curveList[['diff']][['paired']]
-  
-  res <- alphaAdjust(curveList, padj, alpha, cores)
-  pval <- res[['pval']]
-  rho <- res[['rho']]
-  alphastar <- res[['alphastar']]
-  adjpval <- res[['adjpval']]
-  time <- attr(bdObj, "time")
-  sigTime <- bucket(adjpval <= alpha, time)
+
+  # Determine first if we are doing difference of differences
   dod <- ifelse(is.null(innerDiff), FALSE, TRUE)
-  
+
+  ## Currently permutation doesnt exist for dod
+  if (dod & permutation) {
+    warning("Permutation testing does not yet work for difference of difference analysis. Switching to padj='oleson' instead")
+    permutation <- FALSE
+  }
+
+  ## And here we determine significant regions
+  if (permutation) {
+    # do permutation
+    res <- permTest(splitGroups, prs, alpha = alpha, P = Niter, cores = cores,
+                    pAddVar = permAddVar) # in permutation.R
+    obsT <- res[['obst']]
+    nullT <- res[['nullt']]
+
+    # Do I want to return null distribution and vectors?
+    # Do I compute any pvalues here? For now, just significant time
+    time <- attr(bdObj, "time")
+    sigTime <- bucket(res[["sigIdx"]], time)
+
+    # Remove things we don't need (temporary)
+    pval <- NULL
+    rho <- NULL
+    alphastar <- NULL
+    adjpval <- NULL
+  } else {
+    res <- alphaAdjust(curveList, padj, alpha, cores)
+    pval <- res[['pval']]
+    rho <- res[['rho']]
+    alphastar <- res[['alphastar']]
+    adjpval <- res[['adjpval']]
+    time <- attr(bdObj, "time")
+    sigTime <- bucket(adjpval <= alpha, time)
+
+    # Remove things from perm (this is temporary fix)
+    obsT <- NULL
+    nullT <- NULL
+  }
+
   ## maybe consider keeping ancillary data in list, i.e., res.
   structure(class = "bdotsBootObj",
             .Data = list(curveList = curveList,
